#pragma once

#ifndef ASSERTS_IMPLEMENTED
    #ifdef _DEBUG
        #define ASSERTS_IMPLEMENTED 1
    #else
        #define ASSERTS_IMPLEMENTED 0
    #endif
#endif

//      ASSERT:
// - Only evaluates the condition in debug build, in release build it expends to nothing.
// - If the condition is true, it does nothing.
// - If the condition is false, it traps to debugger and prints the message.
//
//      CHECK:
// - Evaluates the condition in both builds.
// - If the condition is true, it does nothing.
// - If the condition is false in debug build,
//   it traps to debugger, prints the message, then executes the ifFailed statement.
// - If the condition is false in release build,
//   it only executes the ifFailed statement, while not even evaluating the rest of the arguments.

// Further ideas on that stuff:
//
// * Make failed ASSERTs visible even without the MSDEV installed on the PC,
//   e.g. MessageBox, or DebugRender, or stderr, or even e-mail report once an hour
//
// * Find a way to disable _particular_ assert in runtime.
//   In real-life scenarios, when something goes wrong, it also probably goes wrong in all the consecutive frames.

#undef ASSERT

#if ASSERTS_IMPLEMENTED

    // It's declared as 'noinline'
    // to minimize the influence on both machine code and local stack of whoever uses ASSERTs/CHECKs
    __declspec(noinline) bool AssertImpl( bool cond, const char* strFile, const char* strFuncName, int iLine, const char* pstrFormat, ... );

	#define ASSERT( cond, msg, ... )                \
        AssertImpl( ( cond ), __FILE__, __FUNCTION__, __LINE__, msg, __VA_ARGS__ )

    #define CHECK( cond, ifFailed, msg, ... )       \
        if( !AssertImpl( ( cond ), __FILE__, __FUNCTION__, __LINE__, msg, __VA_ARGS__ ) ) { ifFailed; }

#else

    #define ASSERT( cond, msg, ... ) __noop

    #define CHECK( cond, ifFailed, msg, ... )   if( !( cond ) ) { ifFailed; }

#endif
