#include "StdAfx.h"
#include "DxWnd.h"

const CPoint CDxWnd::s_MinTrackSize = CPoint( 320, 200 );

CDxWnd::CDxWnd()
{
	m_pD3D = NULL;
	m_pDev = NULL;
//	m_DevParams.Windowed = ( bFullScreen ) ? FALSE : TRUE;
    m_DevParams.Windowed = TRUE;
	m_ttFreq = 0;
	m_dwAnisotropy = 8;
	m_ClearColor = D3DCOLOR_ARGB( 0xff,0,0,0 );

	m_WindowRect = CRect( 0, 0, 0, 0 );
	m_FsSize = CSize( 0, 0 );

    m_bSuppressDeviceOperations = false;
    m_bPreResetCalledAlready = false;
    m_bFirstDeviceError = true;
//    m_bSessionNotificationRegistered = false;
}

CDxWnd::~CDxWnd(void)
{
}

HRESULT CDxWnd::InitD3D()
{
	// Create D3D
	HRESULT hr = S_OK;

	m_pD3D.Attach( Direct3DCreate9( D3D_SDK_VERSION ) );
    if( NULL == m_pD3D )
	{
		UINT res = 	MessageBox( L"It seams you don't have the latest\nDirectX runtime libraries installed.\nThe application can't continue.\n"
			L"Would you like to update the DirectX right now?", L"", MB_YESNO | MB_ICONERROR);
		if(res == IDYES)
			ShellExecute( NULL, L"open", L"http://www.microsoft.com/downloads/details.aspx?familyid=2da43d38-db71-4c1b-bc6a-9b6652cd92a3&displaylang=en", NULL, NULL, SW_SHOWDEFAULT );
		return E_FAIL;
	}

	// Calculate the size for both windowed and full-screen modes
	GetWindowRect( &m_WindowRect );
	DEVMODE dm;	
	EnumDisplaySettings( NULL, ENUM_REGISTRY_SETTINGS, &dm );
	m_FsSize = CSize(dm.dmPelsWidth, dm.dmPelsHeight);
	m_FsRefresh = dm.dmDisplayFrequency;

	// Fill the device parameters
	FillDevParams();

	// Create the D3D device 
	hr = m_pD3D->CreateDevice( 0, D3DDEVTYPE_HAL, m_hWnd,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING,
		&m_DevParams, &m_pDev );
	if(FAILED(hr))
	{
        ReportError( this, MB_ICONERROR, hr, L"Can't create the device." );
        ReleaseComPtr( m_pD3D );
		return hr;
	}


	hr = CreateResources();
	if(FAILED(hr))
	{
        ReportError( this, MB_ICONERROR, hr, L"Can't load the required resources." );
        ReleaseComPtr( m_pDev );
        ReleaseComPtr( m_pD3D );
		return hr;
	}

	InitScene();
	return S_OK;
}

void CDxWnd::FillDevParams()
{
	BOOL bWindowed = m_DevParams.Windowed;
	ZeroMemory( &m_DevParams, sizeof(m_DevParams) );

	D3DDISPLAYMODE d3ddm;
    m_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm );
	m_DevParams.BackBufferFormat = d3ddm.Format;

	m_DevParams.Windowed = bWindowed;
	m_DevParams.hDeviceWindow = m_hWnd;
	m_DevParams.SwapEffect = D3DSWAPEFFECT_DISCARD;
//	m_DevParams.PresentationInterval   = D3DPRESENT_INTERVAL_ONE;
//	m_DevParams.PresentationInterval   = D3DPRESENT_INTERVAL_IMMEDIATE;

	m_DevParams.EnableAutoDepthStencil = TRUE;
	m_DevParams.AutoDepthStencilFormat = D3DFMT_D16;

	if(bWindowed)
	{
		m_DevParams.BackBufferWidth = 0;
		m_DevParams.BackBufferHeight = 0;
	}
	else
	{
		m_DevParams.BackBufferWidth = m_FsSize.cx;
		m_DevParams.BackBufferHeight = m_FsSize.cy;
		m_DevParams.FullScreen_RefreshRateInHz = m_FsRefresh;
	}
}

bool CDxWnd::CheckDevice()
{
	static const DWORD s_msSleepWhenDeviceIsNotReady = 100;

	const HRESULT hr = m_pDev->TestCooperativeLevel();
    switch( hr )
	{
	case D3DERR_DEVICELOST:
	{
		if( m_bFirstDeviceError )
		{
			ATLTRACE( "Device lost.\n" );
			m_bFirstDeviceError = false;
		}
		Sleep( s_msSleepWhenDeviceIsNotReady );
        m_ttFreq=0;
		return false;
	}
	case D3DERR_DEVICENOTRESET:
	{
		m_bFirstDeviceError = true;
		if( DoReset() )
			return true;
		Sleep( s_msSleepWhenDeviceIsNotReady );
        m_ttFreq=0;
		return false;
	}
	case D3DERR_DRIVERINTERNALERROR:
        if( m_bFirstDeviceError )
        {
            ATLTRACE( "D3DERR_DRIVERINTERNALERROR happened while TestCooperativeLevel().\n" );
            m_bFirstDeviceError = false;
        }
        Sleep( s_msSleepWhenDeviceIsNotReady );
        m_ttFreq=0;
        return false;
	default:
		return true;
	}
}

void CDxWnd::DoDrawScene()
{
	QWORD ttNow;
	if( 0 == m_ttFreq )
	{
        m_ttFreq = GetPerfFrequency();
        m_ttLast = GetPerfCounter();
		ttNow = m_ttLast;
	}
	else
		ttNow = GetPerfCounter();

	if( !CheckDevice() )
		return;

	float fltSecsElapsed = static_cast<float>( ttNow - m_ttLast ) / static_cast<float>( m_ttFreq );

	m_pDev->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, m_ClearColor, 1.0f, 0 );
	DrawScene( fltSecsElapsed );
	m_pDev->Present( NULL, NULL, NULL, NULL );

	m_ttLast = ttNow;
}

float CDxWnd::GetCurrentAspectRatio()
{
	int width = m_DevParams.BackBufferWidth;
	int height = m_DevParams.BackBufferHeight;
	if ( height == 0 )
		height = 1;

	return static_cast<float>( width )/static_cast< float >( height );
}

bool CDxWnd::DoReset()
{
	if( false == m_bPreResetCalledAlready )
    {
        OnPreReset();
    }
	
	FillDevParams();
	m_bSuppressDeviceOperations = true;
	HRESULT hr = m_pDev->Reset( &m_DevParams );
	m_bSuppressDeviceOperations = false;
	if( FAILED( hr ) )
	{
		m_bPreResetCalledAlready = true;
		CString msg = L"Reset() failed!\n" + ErrorMessage(hr);
		MessageBox( msg, L"CDxWnd::DoReset", MB_OK );
		msg += L"\n";
		ATLTRACE( msg );
		return false;
	}
	OnPostReset();
    m_bPreResetCalledAlready = false;
	InitScene();
	return true;
}