#pragma once

// Relatively simple Direct3D 9 window class.
// Hit alt+enter for fullscreen // currently excluded for VSTabsDemo
// No resolution is ever changed (unless you'll override FillDevParams),
// so it assumes you have fast enough GPU to render full-screen in its native resolution.
// Only primary display is supported; I had ATI X1650Pro when I first coded this, and now I'm using AMD HD3850, it seams they have severily different device capabilities.
// You may want to do your own WM handling in your derived class, but please don't forget to CHAIN_MSG_MAP(CDxWnd)
class CDxWnd:
    public CWindowImpl<CDxWnd, CWindow, CControlWinTraits>
{
public:
	CDxWnd();
	~CDxWnd();

	DECLARE_WND_CLASS_EX( L"CDxWnd", CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS, -1 );

	BEGIN_MSG_MAP( CDxWnd )
		MESSAGE_HANDLER( WM_CREATE, OnCreate )
		MESSAGE_HANDLER( WM_DESTROY, OnDestroy )
		MESSAGE_HANDLER( WM_PAINT, OnPaint )
		MESSAGE_HANDLER( WM_SIZE, OnSize )
//      MESSAGE_HANDLER( WM_WTSSESSION_CHANGE, OnWtsSessionChange )
//		MESSAGE_HANDLER( WM_GETMINMAXINFO, OnGetMinMaxInfo )
//		MESSAGE_HANDLER( WM_SYSKEYDOWN, OnSysKeyDown );
	END_MSG_MAP()

	inline void Render()
	{
		if( m_bSuppressDeviceOperations )
			Sleep(100);
		else
			DoDrawScene();
	}

    static const CPoint s_MinTrackSize;
protected:
	inline IDirect3DDevice9* GetDevice(){ return m_pDev; }
	__declspec( property( get=GetDevice ) ) IDirect3DDevice9* Device;

	float GetCurrentAspectRatio();

	inline CSize GetCurrentSize()
	{
		return CSize( m_DevParams.BackBufferWidth, m_DevParams.BackBufferHeight );
	}

	D3DCOLOR m_ClearColor;
private:
	// Basic windows messages
	LRESULT OnCreate( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled );
	LRESULT OnDestroy( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled );
	LRESULT OnPaint( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled );
	LRESULT OnSize( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled );
	LRESULT OnGetMinMaxInfo( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled );
//  LRESULT OnWtsSessionChange( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled );
//	LRESULT OnSysKeyDown( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled );
    

	HRESULT InitD3D();

	// Override this if you want to set another device params
	virtual void FillDevParams();

	// Override this to create your D3D resources.
	// The function will be called only once: from WM_CREATE handler.
	virtual HRESULT CreateResources() { return S_OK; }

	// Destroy all your D3D resources.
    // The function will be called only once: from WM_DESTROY event handler.
	virtual void DestroyResources() { }

	// Override this to free all D3DPOOL_DEFAULT resources.
	virtual void OnPreReset() { }

	// Override this to recreate D3DPOOL_DEFAULT resources.
	virtual void OnPostReset() { }

	// Override this to prepare the scene
	// Will be called after resizing window and immediately after each OnPostReset()
	virtual void InitScene() { }

	// Override this to do the actual rendering
	// SecsElapsed will be set to the number of seconds since last call ( common values are between 0.01 and 0.02 )
	virtual void DrawScene( float SecsElapsed ) {  }

	DWORD m_dwAnisotropy;

	// Call IDirect3DDevice9::TestCooperativeLevel and judge whether we should go on with rendering or no
    // Return value 'true' means this very moment is eminently suitable for the rendering.
    bool CheckDevice();

	void DoDrawScene();
	bool DoReset();
	CComPtr< IDirect3D9 > m_pD3D;
	CComPtr< IDirect3DDevice9 > m_pDev;
	D3DPRESENT_PARAMETERS m_DevParams;

	// To calculate elapsed frame time.
	QWORD m_ttFreq, m_ttLast;

	// TODO: convert the following weird boolean values into a kosher DEVICE_STATE enum
    
    // It's set to true while IDirect3DDevice9::Reset is in progress, while the window is minimized.
	bool m_bSuppressDeviceOperations;

    // It's set to true after IDirect3DDevice9::Reset fails, it's reverted to false after OnPostReset().
    bool m_bPreResetCalledAlready;

    // Only used to suppress multiple "Device lost" messages to the debug output.
    bool m_bFirstDeviceError;

    // bool m_bSessionNotificationRegistered;

	// Settings for windowed and FS modes
	CRect m_WindowRect;
	CSize m_FsSize;
	DWORD m_FsRefresh;
};