#include "StdAfx.h"
#include "MainWnd.h"

using NVsTabs::CTabsOwner;

CMainWnd::CMainWnd()
{
}

CMainWnd::~CMainWnd()
{
}

HRESULT CMainWnd::Create( int iCmdShow )
{
	if( NULL == __super::Create( NULL, NULL, L"VSTabs demo" ) )
		return GetLastHr();
	ShowWindow( iCmdShow );
	return S_OK;
}

LRESULT CMainWnd::OnCreate( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
	// I've got no ideas when or why that comctl32/user32 calls will fail,
	// however it's always better to process the error codes..
	m_wndStatus.Attach( ::CreateStatusWindow( WS_CHILD | WS_VISIBLE, L"Application started", m_hWnd, idStatus ) );
	if( NULL == m_wndStatus.m_hWnd )
	{
		ReportError( this, MB_ICONERROR, GetLastHr(), L"CreateStatusWindow() failed." );
		return -1;
	}

	HRESULT hr = NVsTabs::CTabsOwner::Create( this, idTabs );
	if( FAILED( hr ) )
	{
		ReportError( this, MB_ICONERROR, hr, L"CTabsOwner::Create() failed." );
		return -1;
	}

/*
	srand( GetMessageTime() );
	const int nViews = 5;
	for( size_t i=0; i<nViews; i++ )
	{
		CGdiViewWnd *pView = new CGdiViewWnd( this );
		hr = pView->Create( this );
		if( FAILED( hr ) )
		{
			ReportError( this, MB_ICONERROR, hr, L"CGdiViewWnd::Create() failed." );
			return -1;
		}
	}
*/
    BOOL b;
    OnNewView<CBubblesWnd>( 0, 0, 0, b );

    return 0;
}

LRESULT CMainWnd::OnDestroy( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
	PostQuitMessage( 0 );
	return 0;
}

LRESULT CMainWnd::OnSize( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
	if( m_wndStatus.m_hWnd && m_wndStatus.IsWindow() )
		m_wndStatus.SendMessage( WM_SIZE );
    if( wParam != SIZE_MINIMIZED )
	    CTabsOwner::Resize();
	return 0;
}

CRect CMainWnd::GetEffectiveClientRect() const
{
    CRect rc;
	// http://blogs.msdn.com/oldnewthing/archive/2007/11/23/6475511.aspx for more info.
	// GetClientRect / GetWindowRect / ScreenToClient / SubtractRect code doing the same would be much more complex and error-prone.
    int iControls[] = { 0xFFFFFFFF, 0xFFFFFFFF, TRUE, idStatus, TRUE, idTabs, 0, 0 };
    ::GetEffectiveClientRect( m_hWnd, &rc, iControls );
    return rc;
}

CRect CMainWnd::GetTabsRect()
{
	CRect rc;
	CWindow::GetClientRect(rc);
	return rc;
}

CRect CMainWnd::GetViewRect()
{
	return GetEffectiveClientRect();
}

LRESULT CMainWnd::OnCmdAbout(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    CSimpleDialog<IDD_ABOUTBOX, TRUE> aboutbox;
    aboutbox.DoModal( m_hWnd );
    return 0;
}

template<typename TView>
LRESULT CMainWnd::OnNewView( WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled )
{
	TView *pView = new TView(this);	// See CTabViewImpl::OnFinalMessage for the corresponding 'delete'.
	HRESULT hr = pView->Create( this );
	if( FAILED( hr ) )
	{
		delete pView;
		ReportError( this, MB_ICONWARNING, hr, L"Unable to create a new window" );
	}
	return 0;
}

bool CMainWnd::OnIdle()
{
	CWindow* const pActiveView = GetActiveView();
	if( NULL == pActiveView )
		return false;
    const int iBubbles = int( pActiveView->SendMessage( CBubblesWnd::WM_IDLE ) );
	if( iBubbles <= 0 )
		return false;

	// calculate FPS.
	{
		static QWORD ttPrev = 0;

		static double dblTotal = 0;
		static int nTotal = 0;

		if( 0 == ttPrev )
			ttPrev = GetPerfCounter();
		else
		{
			const QWORD ttThis = GetPerfCounter();
			const double FPS = double( GetPerfFrequency() ) / double( ttThis - ttPrev );
			ttPrev = ttThis;

			dblTotal += FPS;
			nTotal++;

			CString strStatusText;
            strStatusText.Format( L"FPS: %.2g current, %.3g average. Total bubbles: %i",
                FPS, dblTotal / nTotal, iBubbles - 1 );
			m_wndStatus.SetWindowText(strStatusText);
		}
	}
	return true;
}
