#pragma once

// TODO: unit test this class just like the Vector3<>

// TODO: find out why the "operator -" doesn't inline in the VS 2005 default "release" configuration.
// This affects performance a lot, unless the project settings are altered.

// The class implements a vector in 2D space
template< typename BaseType>
class __declspec(novtable) Vector2
{
public:
    // Data members
    BaseType x, y;

    static const BaseType valMin;

    inline Vector2(){ x=y=0; };
    inline ~Vector2(){}

    inline Vector2( BaseType _x, BaseType _y )
    {
        x=_x; y=_y;
    };

    inline Vector2( const Vector2& that ){ CopyFrom(that); }
    inline Vector2& operator = ( const Vector2& that ){ CopyFrom(that); return *this; }

    
    explicit inline Vector2( const CPoint& p )
    {
        x = BaseType( p.x );
        y = BaseType( p.y );
    }

    inline void CopyFrom( const Vector2& that )
    {
        x = that.x;
        y = that.y;
    }

    // Addition
    inline Vector2 operator - () const
    {
        Vector2 res;
        res.x = -x;
        res.y = -y;
        return res;
    }

    inline Vector2& operator += ( const Vector2& that )
    {
        x += that.x;
        y += that.y;
        return *this;
    }
    inline Vector2 operator + ( const Vector2& that ) const
    {
        return Vector2(*this) += that;
    }

    inline Vector2& operator -= ( const Vector2& that )
    {
        operator+=(-that);
        return *this;
    }
    inline Vector2 operator - ( const Vector2& that ) const
    {
        return Vector2(*this) -= that;
    }

    // Scale
    inline Vector2& operator *= ( BaseType c )
    {
        x *= c;
        y *= c;
        return *this;
    }
    inline Vector2 operator * ( BaseType that ) const
    {
        return Vector2(*this) *= that;
    }

    // Scale
    inline Vector2& operator /= ( BaseType c )
    {
        ASSERT( abs(c) > valMin, "DISP_E_DIVBYZERO" );
        return operator *= ( BaseType(1.0) / c );
    }

    inline Vector2 operator / ( BaseType that ) const
    {
        return Vector2(*this) /= that;
    }

    // Length
    inline BaseType GetLengthSq() const
    {
        return x*x + y*y;
    }
    __declspec( property( get=GetLengthSq ) ) BaseType LengthSq;

    inline BaseType GetLength() const
    {
        return sqrt( GetLengthSq() );
    }

    inline void SetLength( BaseType newLength )
    {
        const BaseType currLength = GetLength();
        CHECK( abs(currLength) > valMin, return, "Vector3::SetLength() called for a zero-length vector" );
        operator*=( newLength/currLength );
    }
    __declspec( property( get=GetLength, put=SetLength ) ) BaseType Length;

    inline Vector2 Normalize()
    {
        Length = 1;
        return *this;
    }

    inline Vector2 ScaleBy( const Vector2& that ) const
    {
        return Vector2( x * that.x, y * that.y );
    }
};

template <typename T>
const T Vector2<T>::valMin = std::numeric_limits<T>::min();

typedef Vector2<double> Vector2D;
typedef Vector2<float> Vector2F;
