#pragma once

struct ColoringParams
{
    // Bubbles coloring modes
    enum COLORING_MODE
    {
		COLOR_CONSTANT,                   // All bubbles are of the same color (possibly affected by the highlighting)
		COLOR_SELF,                       // Unique color of each bubble
		COLOR_NEIGBOURS_SNAPSHOT,         // Proportional to the momentary count of neighbors
		COLOR_NEIGBOURS_FLOATING_AVERAGE  // Proportional to the average count of neighbors during the last several frames
    }
    coloringMode;

    // Color for COLOR_CONSTANT coloring mode.
    DWORD constantSolor;

    // Highlight mode.
    // Unless this is set to HIGHLITE_NOTHING,
    // the bubble's resulting color's saturation and value will be overridden by the highlight procedure.
    enum HIGHLIGHT_MODE
    {
		HIGHLITE_NOTHING,
		HIGHLITE_ENERGY_SNAPSHOT,
		HIGHLITE_ENERGY_AVERAGE
    }
    highlitMode;
};

struct ClickActions
{
    // Action initiated by mouse clicks..
    enum CLICK_ACTION
    {
        ADD_SINGLE,
        ADD_AREA,

        DEL_SINGLE,
        DEL_AREA,

        PAINT_SINGLE,
        PAINT_AREA,

        HEAT_SINGLE,
        HEAT_AREA
    }
    lmb, rmb;   //< for left and right mouse buttons, respectively

    double areaSize;
    DWORD paintColor;
};

__interface IBubbleSettings
{
    HRESULT GetColoring( ColoringParams* params );
    HRESULT SetColoring( const ColoringParams* params );

    HRESULT GetClickActions( ClickActions* params );
    HRESULT SetClickActions( const ClickActions* params );
};
