#include "stdafx.h"
#include "SettingsWindow.h"
#include "IBubbleSettings.h"

CBubbleSettingsWnd::CBubbleSettingsWnd()
{
    m_pBubbles = NULL;
}

CBubbleSettingsWnd::~CBubbleSettingsWnd()
{
}

/*
LRESULT CBubbleSettingsWnd::OnButtonsBindingsChanged( WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled )
{
    return 0;
}
*/

HRESULT CBubbleSettingsWnd::Create( CWindow* pWndParent, IBubbleSettings* pBubbles )
{
	m_pBubbles = pBubbles;
    HWND w = __super::Create( pWndParent->m_hWnd );
    if( NULL == w ) return GetLastHr();
    return S_OK;
}

void CBubbleSettingsWnd::OnWindowPosChanged( LPWINDOWPOS lpWndPos )
{
    SetMsgHandled(FALSE);
    if( !IsWindowVisible() )
        GetParent().SetFocus();
}

BOOL CBubbleSettingsWnd::OnInitDialog( CWindow wndFocus, LPARAM lInitParam )
{
/*
    m_cbLeftClick = GetDlgItem(IDC_LEFTCLICK);
    m_cbRightClick = GetDlgItem(IDC_RIGHTCLICK);
    m_cbHighlite = GetDlgItem(IDC_HIGHLITE);

    m_cbHighlite.AddString( L"Nothing" );
    m_cbHighlite.AddString( L"Average Energy" );
    m_cbHighlite.SetCurSel( 1 );
*/
	m_cbColor = GetDlgItem( IDC_COLOR_MODE );
	m_cbColor.AddString( L"Constant color" );
	m_cbColor.AddString( L"Self color" );
	m_cbColor.AddString( L"Structure (momentary)" );
	m_cbColor.AddString( L"Structure (average)" );
	

	m_cbHighlite = GetDlgItem( IDC_HIGHLITE );
	m_cbHighlite.AddString( L"Nothing" );
	m_cbHighlite.AddString( L"Energy (momentary)" );
	m_cbHighlite.AddString( L"Energy (average)" );

	UpdateData( FALSE );
	

    return FALSE;   //< leave the keyboard focus where it was
}

void CBubbleSettingsWnd::ShowHideWholePanel( bool bShow )
{
    if( bShow )
        ::AnimateWindow( m_hWnd, 200, AW_HOR_POSITIVE | AW_SLIDE );
    else
        ::AnimateWindow( m_hWnd, 200, AW_HOR_NEGATIVE | AW_SLIDE | AW_HIDE );
}

/*
void CBubbleSettingsWnd::PaintBubblesSame( UINT uNotifyCode, int nID, CWindow wndCtl )
{
    HRESULT hr = m_pBubbles->PaintAllBubbles( IBubbleSettings::PAINT_ALL_SAME );
    ASSERT( SUCCEEDED(hr), "IBubbleSettings::PaintAllBubbles failed:\n%S", LPCTSTR(ErrorMessage(hr)));
}

void CBubbleSettingsWnd::PaintBubblesRandom( UINT uNotifyCode, int nID, CWindow wndCtl )
{
    HRESULT hr = m_pBubbles->PaintAllBubbles( IBubbleSettings::PAINT_ALL_RANDOM );
    ASSERT( SUCCEEDED(hr), "IBubbleSettings::PaintAllBubbles failed:\n%S", LPCTSTR(ErrorMessage(hr)));
}

void CBubbleSettingsWnd::PaintBubblesDomains( UINT uNotifyCode, int nID, CWindow wndCtl )
{
    HRESULT hr = m_pBubbles->PaintAllBubbles( IBubbleSettings::PAINT_ALL_DOMAINS );
    ASSERT( SUCCEEDED(hr), "IBubbleSettings::PaintAllBubbles failed:\n%S", LPCTSTR(ErrorMessage(hr)));
}

void CBubbleSettingsWnd::OnHighliteCloseup( UINT uNotifyCode, int nID, CComboBoxT<CWindow> wndCtl )
{
    const int cs = wndCtl.GetCurSel();
    switch( cs )
    {
    case 0:
        m_pBubbles->HighliteBubbles( IBubbleSettings::HIGHLITE_NOTHING );
        return;
    case 1:
        m_pBubbles->HighliteBubbles( IBubbleSettings::HIGHLITE_AVERAGE_ENERGY );
        return;
    }
}
*/

BOOL CBubbleSettingsWnd::UpdateData( BOOL bSaveAndValidate )
{
	ColoringParams coloring;
	HRESULT hr = m_pBubbles->GetColoring(&coloring);
	CHECK( SUCCEEDED(hr), return FALSE, "IBubbleSettings::GetColoring failed\n%S", LPCTSTR(ErrorMessage(hr)) );

	ASSERT( int(coloring.coloringMode) < m_cbColor.GetCount(), "COLORING_MODE: inconsistency with the UI" );
	if( bSaveAndValidate )
		coloring.coloringMode = (ColoringParams::COLORING_MODE)( m_cbColor.GetCurSel() );
	else
		m_cbColor.SetCurSel( coloring.coloringMode );

	ASSERT( int(coloring.highlitMode) < m_cbHighlite.GetCount(), "HIGHLIGHT_MODE: inconsistency with the UI" );
	if( bSaveAndValidate )
		coloring.highlitMode = (ColoringParams::HIGHLIGHT_MODE)( m_cbHighlite.GetCurSel() );
	else
		m_cbHighlite.SetCurSel( coloring.highlitMode );

	if( bSaveAndValidate )
	{
		hr = m_pBubbles->SetColoring( &coloring );
		CHECK( SUCCEEDED(hr), return FALSE,
			"IBubbleSettings::SetColoring failed\n%S", LPCTSTR(ErrorMessage(hr)) );
	}

	return TRUE;
}

void CBubbleSettingsWnd::OnCaptureChanged( CWindow wnd )
{
	GetParent().SetFocus();	
}