#include "stdafx.h"
#include "SoapBubbles.h"

// TODO: rename this .CPP into SoapBubbles_Render

CSoapBubblesRender::CSoapBubblesRender()
{
    m_pDevice = NULL;
    const ColoringParams defaultParams = 
    {
        ColoringParams::COLOR_NEIGBOURS_SNAPSHOT,  // coloringMode
        D3DCOLOR_RGBA( 0x9A, 0xCD, 0x32, 255 ),    // constantSolor: "yellowgreen"
        ColoringParams::HIGHLITE_ENERGY_AVERAGE,   // highlightMode: average energy between the frames
    };
    m_BubblesColoring = defaultParams;
}

CSoapBubblesRender::~CSoapBubblesRender()
{
}

/*
void CSoapBubblesSim::BubbleProc_PaintStructure( BubbleSim& e1, BubbleSim& e2 ) const
{
    const double dblDistance = (e1.pos - e2.pos).Length;
    const double dblThreshold = double( e1.size + e2.size ) * 0.5;

    if( dblDistance > dblThreshold + 1.0 )
        return;

    tTmpMap::CPair* p = m_tmpNeighboursMap.Lookup(&e1);
    if( NULL == p ) m_tmpNeighboursMap[&e1] = 1; else p->m_value++;

    p = m_tmpNeighboursMap.Lookup(&e2);
    if( NULL == p ) m_tmpNeighboursMap[&e2] = 1; else p->m_value++;
}
*/

/*
HRESULT CSoapBubblesSim::PaintAllBubbles( IBubbleSettings::PAINT_ALL_MODE mode )
{
    if( m_Bubbles.GetBubblesCount() <= 0 )
        return S_FALSE;

    switch( mode )
    {
    case IBubbleSettings::PAINT_ALL_SAME:
        {
            const BYTE cColor = rand();
            POSITION pos;
            for( m_Bubbles.GetAllBubbles( pos ); NULL != pos; )
            {
                BubbleSim &e = m_Bubbles.GetValueAndNextPos( pos );
                e.selfColor_deprecated = cColor;
            }
            return S_OK;
        }
    case IBubbleSettings::PAINT_ALL_RANDOM:
        {
            POSITION pos;
            for( m_Bubbles.GetAllBubbles( pos ); NULL != pos; )
            {
                BubbleSim &e = m_Bubbles.GetValueAndNextPos( pos );
                e.selfColor_deprecated = BYTE(rand());
            }
            return S_OK;
        }
    case IBubbleSettings::PAINT_ALL_DOMAINS:
        {
            // Copy the stuff to a local buffer.
            POSITION pos;
            const size_t nBubbles = m_Bubbles.GetAllBubbles( pos );
            if( !nBubbles )
                return S_FALSE;

            m_tmpNeighboursMap.InitHashTable( UINT( nBubbles / 0.75 ), true );
            ProcessInteractions( &CSoapBubblesSim::BubbleProc_PaintStructure );

            size_t tmpMaxNeighbours = 0;
            pos = m_tmpNeighboursMap.GetStartPosition();
            while( pos != NULL )
            {
                tmpMaxNeighbours = std::max( tmpMaxNeighbours,
                    m_tmpNeighboursMap.GetValueAt( pos ) );
                m_tmpNeighboursMap.GetNext( pos );
            }

            for( m_Bubbles.GetAllBubbles( pos ); NULL != pos; )
            {
                BubbleSim &e = m_Bubbles.GetValueAndNextPos( pos );
                const size_t nNeighbours = m_tmpNeighboursMap[&e];
                const float fSelfColor = float( nNeighbours ) / float( tmpMaxNeighbours + 1 );
                e.selfColor_deprecated = BYTE( 255.0f * fSelfColor );
            }

            m_tmpNeighboursMap.RemoveAll();
            return S_OK;
        }
    }

    ASSERT( 0, "Unknown IBubbleSettings::PAINT_ALL_MODE value" );
    return E_FAIL;
}
*/

HRESULT CSoapBubblesRender::BufferCreate( IDirect3DDevice9* pDevice, D3DPOOL Pool )
{
    const size_t szCurr = m_vb.GetCount();
    const size_t szNeeded = m_Bubbles.GetBubblesCount();

    if( szCurr >= szNeeded )
        return S_FALSE;

    HRESULT hr = S_OK;
    const size_t s_RoundMask = 0x3FF;   // = 1024 - 1
    const size_t szToAlloc = ( ( szNeeded - 1 ) | s_RoundMask ) + 1;
    if( szCurr > 0 )
    {
        hr = m_vb.Destroy();
        if(FAILED(hr)) return hr;
    }
    hr = m_vb.Create( pDevice, szToAlloc, Pool );
    if(FAILED(hr)) return hr;

    m_pDevice = pDevice;
    m_pool = Pool;
    return S_OK;
}

HRESULT CSoapBubblesRender::BufferDestroy()
{
    return m_vb.Destroy();
}

HRESULT CSoapBubblesRender::BufferUpdate()
{
    HRESULT hr = BufferCreate( m_pDevice, m_pool );
    if(FAILED(hr)) return hr;

    POSITION pos;
    const size_t nBubbles = m_Bubbles.GetAllBubbles( pos );
    {
        CVertexBufferLock<BubbleRender> lock( m_vb, 0, nBubbles );
        for( size_t i=0; i<nBubbles; i++ )
            lock[i] = PrepareBubbleForRender( m_Bubbles.GetValueAndNextPos(pos) );
    }

    hr = m_vb.SetStreamSource( m_pDevice, 0 );
    if(FAILED(hr)) return hr;

    return S_OK;
}

HRESULT CSoapBubblesRender::BufferSetStreamSource( UINT nStream )
{
    return m_vb.SetStreamSource( m_pDevice, 0 );
}

CSoapBubblesRender::BubbleRender CSoapBubblesRender::PrepareBubbleForRender( const CSoapBubblesSim::BubbleSim& bs )
{
    BubbleRender r;
    r.pos = Vector3F( float(bs.pos.x), float(bs.pos.y), 0 );
    r.pxSize = float( bs.size );

	BYTE structure = bs.nNeighbours;

    DWORD DiffuseColor = 0;
	float fHue = -1;

    switch( m_BubblesColoring.coloringMode )
    {
    case ColoringParams::COLOR_CONSTANT:
        DiffuseColor = m_BubblesColoring.constantSolor;
        break;
    case ColoringParams::COLOR_SELF:
		DiffuseColor = bs.selfColor;
		break;
    case ColoringParams::COLOR_NEIGBOURS_SNAPSHOT:
		fHue = ( bs.nNeighbours <= 6 )  ?  0.7f - float( bs.nNeighbours ) * 0.12f  :  0.9f;
		break;
    case ColoringParams::COLOR_NEIGBOURS_FLOATING_AVERAGE:
		ASSERT( 0, "Unsupported COLORING_MODE value" );
		break;
    default:
        ASSERT( 0, "Unknown COLORING_MODE value" );
    }

	float fHighlite = -1.0f;
	switch( m_BubblesColoring.highlitMode )
	{
	case ColoringParams::HIGHLITE_NOTHING:
		break;
	case ColoringParams::HIGHLITE_ENERGY_AVERAGE:
		fHighlite = float( std::clamp<ptrdiff_t>( bs.energyFloatingAverage, 0, 0xFF00 ) / float( 0xFF00 ) );
		break;
	case ColoringParams::HIGHLITE_ENERGY_SNAPSHOT:
		fHighlite = float( bs.energyCurrentLevel ) / 255.0f;
		break;
	default:
		ASSERT( 0, "Unknown HIGHLIGHT_MODE value" );
	}

	if( fHighlite >= 0.0f )
	{
		// Highlight
		if( fHue < 0 )
			fHue = HueFromRgb( DiffuseColor );

		fHighlite += 2.0f / 3.0f;
		if( fHighlite <= 1.0f )
			DiffuseColor = RgbFromHsv_f( fHue, 1.0f, fHighlite );
		else
			DiffuseColor = RgbFromHsv_f( fHue, fHighlite - 1.0f, 1.0f );
	}
	else
	{
		// Don't highlight
		if( fHue >= 0 )
			DiffuseColor = RgbFromHsv_f( fHue, 1.0f, 1.0f );
	}

	r.diffuseColor = DiffuseColor;

    return r;
}

HRESULT CSoapBubblesModel::AddRandomBubbles( size_t nBubbles, CSize sz )
{
    if( !nBubbles )
        return S_FALSE;

    const Vector2D vTopLeft( -sz.cx / 2, -sz.cy / 2 );
    const Vector2D vSize( sz.cx, sz.cy );

    for( size_t n=0; n<nBubbles; n++ )
    {
        BubbleSim b;
        const Vector2D vRand( ( rand() & 0x1FFF ) / double( 0x1FFF ), ( rand() & 0x1FFF ) / double( 0x1FFF ) );
        b.pos = vTopLeft + vRand.ScaleBy( vSize );
        b.size = s_Size;
        b.selfColor = RgbFromHsv( rand(), 255, 255 );
        m_Bubbles.Add(b);
    }
    return S_OK;
}

HRESULT CSoapBubblesModel::MouseOperation( UINT vk, Vector2D worldPos )
{
    switch( vk )
    {
    case VK_LBUTTON:
		{
			BubbleSim b;
			b.pos = worldPos;
			b.size = s_Size;
			b.selfColor = RgbFromHsv( rand(), 255, 255 );
			m_Bubbles.Add(b);
		}
        return S_OK;

    case VK_RBUTTON:
        RemoveAround( worldPos, 15.0 );
        return S_OK;
    }
    ASSERT( 0, "Unsupported mouse operation" );
	return E_NOTIMPL;
}