#pragma once

// This class initializes and stores a various GDI resources involved in the painting.
// Almost every class that paints something holds a constant reference to CGdiResources.
// The only instance of this class is owned by CMainWnd.
class CGdiResources
{
public:
	// This class really owns the GDI resources.
	// The only instance of this one is stored in the static variable
	class Resources
	{
	public:
		Resources();

		HRESULT Create();
		HRESULT Destroy();

		HBITMAP hTranspBrushBitmap;
		HBRUSH hTransparencyBrush;

		HCURSOR curHand, curArrow;

		HCURSOR curSizeAll, curSizeV, curSizeH;

	private:
		// Disable copying
		Resources( const Resources& that );
		Resources& operator=( const Resources& that );
	};

	CGdiResources();

	HRESULT Create();

	// Only return const pointer,
    // so the compiler will check the clients are not modifying the data.
    inline const Resources* operator->(){ return &m_sResources; }

	~CGdiResources();
private:
	// Set to true if this instance of the class was initialized OK
	bool m_bInitDone;

	// Count of successful CGdiResources::Create() calls
	static size_t m_sInitCount;

	// Actual resources
	static Resources m_sResources;
};
