#include "stdafx.h"
#include "ImageWnd.h"
#include "..\resource.h"
#include "GdiResources.h"

//#define LOCK() CComCritSecLock<CComAutoCriticalSection> _Lock(m_cs);

const DDARGB CImageWnd::s_EmptyColor = { 255, 0, 255, 0x66 };

const CImageWnd::ZoomLevel CImageWnd::s_ZoomLevels[] = 
{//  MUL | DIV
      1,    4,   // 25%
      1,    3,
      1,    2,
      1,    1,   // 100%, this one should be the default
      2,    1,
      3,    1,
      4,    1,
      8,    1,
     16,    1    // 1600%
};

const int CImageWnd::s_DefaultZoomLevel = 3;

CImageWnd::CImageWnd( NVsTabs::CTabsOwner* pOwner ):
	NVsTabs::CTabViewWindowImpl<CImageWnd>( pOwner )
{
    m_hMemBitmap = NULL;
    m_pMemBitmapData = NULL;
    m_memSize.SetSize(0,0);

    m_bHScroll=m_bVScroll=false;

    m_hMemDC = NULL;
    m_hPrevBitmap = NULL;

    m_ZoomLevel = s_DefaultZoomLevel;
    m_bMouseTracking = m_bMouseDrag = false;

    m_clientTopTeft = CPoint(0);
}

HRESULT CImageWnd::Create( CWindow* pParent )
{
	HRESULT hr = m_res.Create();
	if(FAILED(hr))
	{
		ReportError( this, MB_ICONWARNING, hr, L"Unable to create required GDI resources." );
		return hr;
	}

	static int iSN = 1;
	CString strName;
	strName.Format( L"Image Window #%i", iSN );
	iSN++;

	if( NULL == __super::Create( pParent->m_hWnd, NULL, strName, WS_CHILD ) )
		return GetLastHr();

	// Test..
	if(1)
	{
		InitImage(CSize(256,256));
		DDARGB line[256];
		CPoint pt;
		for( pt.y=0; pt.y<256; pt.y++ )
		{
			for( pt.x=0; pt.x<256; pt.x++ )
			{
				const DDARGB c = { BYTE( pt.x ), 128, BYTE( pt.y ), BYTE( rand() ) };
				line[pt.x] = c;
			}
			SetBitmapLine( pt.y, line );
		}
	}

	return S_OK;
}

CImageWnd::~CImageWnd()
{
    // Delete old GDI resources, if any
    if(m_hMemDC)
    {
        if(m_hPrevBitmap)
            SelectObject(m_hMemDC, m_hPrevBitmap);
        DeleteDC(m_hMemDC);
    }
    if(m_hMemBitmap)
        DeleteObject(m_hMemBitmap);
};

HRESULT CImageWnd::InitImage(CSize sz)
{
    // Delete old GDI resources, if any
    if(m_hMemDC)
    {
        if(m_hPrevBitmap)
        {
            SelectObject(m_hMemDC, m_hPrevBitmap);
            m_hPrevBitmap = NULL;
        }
        DeleteDC(m_hMemDC);
        m_hMemDC = NULL;
    }
    if(m_hMemBitmap)
    {
        if(!DeleteObject(m_hMemBitmap))
            return GetLastHr();
        m_hMemBitmap = NULL;
    }

    m_memSize.SetSize(0,0);

    BITMAPINFO bmi;
    ZeroMemory(&bmi, sizeof(BITMAPINFO));
    bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bmi.bmiHeader.biWidth = sz.cx;
    bmi.bmiHeader.biHeight = sz.cy;
    bmi.bmiHeader.biPlanes = 1;
    bmi.bmiHeader.biBitCount = 32;
    bmi.bmiHeader.biCompression = BI_RGB;

    void *pvBits;
    m_hMemBitmap = CreateDIBSection(m_hMemDC, &bmi, DIB_RGB_COLORS, &pvBits, NULL, 0);
    if( NULL == m_hMemBitmap )
        return GetLastHr();

    m_pMemBitmapData = reinterpret_cast<RGBQUAD*>(pvBits);
    m_memSize = sz;
    FillNewImage();
    BOOL b;
    OnSize(WM_SIZE, 0, 0, b);
    InvalidateRect(NULL, TRUE);
    return S_OK;
}

void CImageWnd::FillNewImage()
{
    const RGBQUAD toFill = PremultiplyAlpha(s_EmptyColor);
    RGBQUAD *pPixel = m_pMemBitmapData;
    CPoint pt;
    for( pt.y=0; pt.y < m_memSize.cy; pt.y++ )
    {
        for( pt.x=0; pt.x < m_memSize.cx; pt.x++ )
        {
            *pPixel = toFill;
            pPixel++;
        }
    }
}

BOOL CImageWnd::AlphaBlend(HDC dcDest, CRect& rcDest, HDC dcSrc, CRect rcSrc)
{
    // Maybe there's a better way to compensate MilDiv arithmetic errors when zoom != N * 100%
    rcSrc.left = std::max( rcSrc.left, 0L );
    rcSrc.top = std::max( rcSrc.top, 0L );
    rcSrc.right = std::min( rcSrc.right, m_memSize.cx );
    rcSrc.bottom = std::min( rcSrc.bottom, m_memSize.cy );

    BLENDFUNCTION bf;
    bf.BlendOp = AC_SRC_OVER;
    bf.BlendFlags = 0;
    bf.SourceConstantAlpha = 0xFF;  // 0x00 (transparent) through 0xFF (opaque)
                                    // want bitmap alpha, so no constant alpha.
    bf.AlphaFormat = AC_SRC_ALPHA;  // Use bitmap alpha

    return ::AlphaBlend(dcDest,
        rcDest.left, rcDest.top, rcDest.Width(), rcDest.Height(),
        dcSrc,
        rcSrc.left, rcSrc.top, rcSrc.Width(), rcSrc.Height(),
        bf);
}

HRESULT CImageWnd::SetBitmapLine( int iLine, const DDARGB* pPixels )
{
    if( iLine < 0 || iLine >= m_memSize.cy )
        return E_INVALIDARG;
    if( NULL == pPixels )
        return E_POINTER;

    // Update a bitmap
    const int memLine = ( m_memSize.cy - iLine - 1 );
    RGBQUAD *pDest = m_pMemBitmapData + memLine * m_memSize.cx;
    for( LONG x=0; x < m_memSize.cx; x++ )
    {
        *pDest = PremultiplyAlpha(*pPixels);
        pDest++;
        pPixels++;
    }
    const CRect rcMem( CPoint( 0, iLine ), CSize( m_memSize.cx, 1 ) );
    const CRect rcClient = clientFromMemRect( rcMem );
    InvalidateRect( &rcClient, FALSE );

    return S_OK;
}

CSize CImageWnd::memFromClientSz( CSize clientSz )
{
    const ZoomLevel z = s_ZoomLevels[ m_ZoomLevel ];
    return CSize
    (
        MulDiv( clientSz.cx, z.iDiv, z.iMul ),
        MulDiv( clientSz.cy, z.iDiv, z.iMul )
    );
}

CSize CImageWnd::clientFromMemSz( CSize memSz )
{
    const ZoomLevel z = s_ZoomLevels[ m_ZoomLevel ];
    return CSize
    (
        MulDiv( memSz.cx, z.iMul, z.iDiv ),
        MulDiv( memSz.cy, z.iMul, z.iDiv )
    );
}

CRect CImageWnd::clientFromMemRect( CRect rc )
{
    return CRect( clientFromMemPt( rc.TopLeft() ), clientFromMemPt( rc.BottomRight() ) );
}

CPoint CImageWnd::memFromClientPt( CPoint clientPt )
{
    return memFromClientSz( clientPt - m_clientTopTeft );
}

CPoint CImageWnd::clientFromMemPt( CPoint memPt )
{
    return m_clientTopTeft + clientFromMemSz( memPt );
}

LRESULT CImageWnd::OnPaint( UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
{
    if( m_memSize.cx <=0 || m_memSize.cy <=0 )
    {
        bHandled = FALSE;
        return 0;
    }

    PAINTSTRUCT ps;
    HDC dc = BeginPaint(&ps);

    // Calculate dirty region in client coordinates
    const CRect clientImage( m_clientTopTeft, clientFromMemSz( m_memSize ) );
    CRect clientDirty = IntersectRect(m_ClientRect, clientImage);
    clientDirty = IntersectRect( clientDirty, ps.rcPaint );
    if( clientDirty.IsRectEmpty() )
    {
        EndPaint(&ps);
        return 0;
    }

    // Deal with rounding errors this way
    ExpandDirtyRect(clientDirty);

    // Draw a background bitmap, setting the brush origin point.
    CRect clientBgRect = clientDirty;
    SetBrushOrgEx( dc, m_clientTopTeft.x & 0xF, m_clientTopTeft.y & 0xF, NULL );
    FillRect( dc, clientDirty, m_res->hTransparencyBrush );

    // Draw a bitmap, finally
    CRect memDirty( memFromClientPt( clientDirty.TopLeft() ), memFromClientPt( clientDirty.BottomRight() ) );
    if( !m_hMemDC )
    {
        m_hMemDC = ::CreateCompatibleDC( dc );
        m_hPrevBitmap = (HBITMAP)(SelectObject( m_hMemDC, m_hMemBitmap ) );
    }
    AlphaBlend( dc, clientDirty, m_hMemDC, memDirty );

    EndPaint( &ps );
    return 0;
}

LRESULT CImageWnd::OnChar( UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
{
    switch(wParam)
    {
    case VK_ADD:
    case '+':
        SetZoomLevel( m_ZoomLevel + 1, m_ClientRect.CenterPoint() );
        break;
    case VK_SUBTRACT:
    case '-':
        SetZoomLevel( m_ZoomLevel - 1, m_ClientRect.CenterPoint() );
        break;
    case VK_DIVIDE:
    case '/':
        SetZoomLevel( s_DefaultZoomLevel, m_ClientRect.CenterPoint() );
        break;
    default:
        bHandled = FALSE;
    }
    return 0;
}

LRESULT CImageWnd::OnMButtonUp( UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
{
    SetZoomLevel(s_DefaultZoomLevel, CPoint( lParam ) );
    return 0;
}

LRESULT CImageWnd::OnMouseWheel( UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
{
    if( 0 == ( MK_CONTROL & LOWORD( wParam ) ) )
    {
        bHandled = FALSE;
        return 0;
    }
    const int wheelAmount = ( signed __int16 )( HIWORD( wParam ) );
    if( wheelAmount > 0 )
        SetZoomLevel( m_ZoomLevel + 1, CPoint(lParam) );
    if( wheelAmount < 0 )
        SetZoomLevel( m_ZoomLevel - 1, CPoint(lParam) );
    return 0;
}

void CImageWnd::SetZoomLevel( int iNewZoomLevel, CPoint clientAhchor )
{
    const int iTotalZoomLvls = sizeof( s_ZoomLevels ) / sizeof( s_ZoomLevels[0] );
    if( iNewZoomLevel < 0 )
        iNewZoomLevel=0;
    if( iNewZoomLevel >= iTotalZoomLvls ) 
        iNewZoomLevel = iTotalZoomLvls-1;
    if( iNewZoomLevel == m_ZoomLevel)
        return;

	const CPoint memAnchor = memFromClientPt( clientAhchor );
    m_ZoomLevel = iNewZoomLevel;
    BOOL btmp;

    OnSize( WM_SIZE, 0, 0, btmp );

	const CPoint clientNewAnchor = clientFromMemPt( memAnchor );

	CRect clientRect = m_ClientRect;
	clientRect.OffsetRect( clientNewAnchor  - clientAhchor );
	CRect clientImage( m_clientTopTeft, clientFromMemSz( m_memSize ) );
	clientRect = PlaceRectInsideOrCenter( clientRect, clientImage );
	m_clientTopTeft = clientImage.TopLeft() - clientRect.TopLeft();

    InvalidateRect( NULL, TRUE );
}

LRESULT CImageWnd::OnWmMouseLeave( UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
{
    m_bMouseTracking = false;
    m_bMouseDrag = false;
    SetCursor( m_res->curArrow );
    return 0;
}

LRESULT CImageWnd::OnWmMouseUp( UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
{
    m_bMouseDrag = false;
    SetCursor( m_res->curArrow );
    return 0;
}

CRect CImageWnd::PlaceRectInsideOrCenter(const CRect& RectToPlace, const CRect& OuterArea)
{
    CRect rc = RectToPlace;

    if( rc.Height() >= OuterArea.Height() )  // Not fit - just center
        rc.OffsetRect( 0, ( OuterArea.CenterPoint() - rc.CenterPoint() ).cy );
    else
    {
        if( rc.top < OuterArea.top )             // too high
            rc.MoveToY( OuterArea.top );
        else if( rc.bottom > OuterArea.bottom )  // too low
            rc.MoveToY( OuterArea.bottom - rc.Height() );
    }

    if( rc.Width() >= OuterArea.Width() )         // Not fit - just center
        rc.OffsetRect( ( OuterArea.CenterPoint() - rc.CenterPoint() ).cx, 0 );
    else
    {
        if( rc.left < OuterArea.left )          // too left
            rc.MoveToX( OuterArea.left );
        else if( rc.right > OuterArea.right )   // too right
            rc.MoveToX( OuterArea.right - rc.Width() );
    }

    return rc;
}

LRESULT CImageWnd::OnSize( UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
{
    GetClientRect( &m_ClientRect );

    const CSize clientSz = clientFromMemSz( m_memSize );
    const CRect clientImage( m_clientTopTeft, clientFromMemSz( m_memSize ) );

    CRect clientRect = PlaceRectInsideOrCenter( m_ClientRect, clientImage );
    m_clientTopTeft = clientImage.TopLeft() - clientRect.TopLeft();
    return 0;
}

void CImageWnd::ExpandDirtyRect(CRect& rc)
{
    if( m_ZoomLevel <= s_DefaultZoomLevel )
        return;

    int iMagnification = s_ZoomLevels[m_ZoomLevel].iMul;
    int i = 0;
    
    i = ( rc.left - m_clientTopTeft.x ) % iMagnification;
    if( i ) rc.left -= i;
    i = ( rc.right - m_clientTopTeft.x ) % iMagnification;
    if( i ) rc.right += ( iMagnification - i );


    i = ( rc.top - m_clientTopTeft.y ) % iMagnification;
    if( i ) rc.top -= i;
    i = ( rc.bottom - m_clientTopTeft.y ) % iMagnification;
    if( i ) rc.bottom += ( iMagnification - i );
}

LRESULT CImageWnd::OnWmMouseMove( UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
{
    if( !m_bMouseTracking )
    {
        TRACKMOUSEEVENT tme;
        tme.cbSize = sizeof( TRACKMOUSEEVENT );
        tme.dwFlags = TME_LEAVE;
        tme.hwndTrack = m_hWnd;
        TrackMouseEvent( &tme );
        m_bMouseTracking = true;
        SetCursor( m_res->curArrow );
    }
    if(!m_bMouseDrag)
        return 0;

    const CPoint pt = CPoint(lParam);
    const CPoint ptShift = pt - m_ptMouseDown;

    // Calculate the rectangles
    CRect clientImage( m_clientTopTeft, clientFromMemSz( m_memSize ) );
    CRect clientRect = m_ClientRect;
    clientRect.OffsetRect( -ptShift );

    // Limit the maximum shift
    clientRect = PlaceRectInsideOrCenter( clientRect, clientImage );

    // Check did we really move somewhere
    const CPoint newClientTopTeft = clientImage.TopLeft() - clientRect.TopLeft();
    if( m_clientTopTeft != newClientTopTeft )
    {
        const CSize szScroll = newClientTopTeft-m_clientTopTeft;
        ScrollWindowEx( szScroll.cx, szScroll.cy, SW_INVALIDATE );
        m_clientTopTeft = newClientTopTeft;
    }
    m_ptMouseDown = pt;
    return 0;
}

LRESULT CImageWnd::OnWmMouseDown( UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
{
    const CPoint pt(lParam);
    const CRect clientImage( m_clientTopTeft, clientFromMemSz( m_memSize ) );
    if( clientImage.PtInRect( pt ) )
    {
        CRect clientRect;
        GetClientRect(&clientRect);

        CRect unionRect;
        unionRect.UnionRect(clientImage, clientRect);
        if( unionRect != clientRect)
        {
            m_bMouseDrag = true;
            bool bVertical = ( clientImage.Height() > clientRect.Height() );
            bool bHorizontal = ( clientImage.Width() > clientRect.Width() );
            if( bVertical && bHorizontal )
                SetCursor( m_res->curSizeAll );
            else
                if(bVertical)
                    SetCursor( m_res->curSizeV );
                else
                    SetCursor( m_res->curSizeH );
            m_ptMouseDown = pt;
        }
    }

    return 0;
}
