#include "stdafx.h"

__declspec(noinline) CString ErrorMessage(HRESULT hr)
{
    CString res;

    // System error
    LPVOID lpMsgBuf;
    if( FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
        NULL, hr, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ),
        (LPTSTR) &lpMsgBuf, 0, NULL ) )
    {
        res.Format( L"%s", lpMsgBuf );
        LocalFree( lpMsgBuf );
        return res;
    }

    // Direct3D error, if the header is included
#ifdef DIRECT3D_VERSION
    LPCTSTR strDX = DXGetErrorString(hr);
    if(strDX)
    {
        CString Res;
        Res.Format(L"%s (%s)", DXGetErrorDescription(hr), strDX);
        return strDX;
    }
#endif

    res.Format(L"Unknown error 0x%.8X", hr);
    return res;
}

void ReportError( CWindow* pWnd, UINT uMbIconType, HRESULT hr, LPCTSTR pstrFormat, ... )
{
    CString str;

    va_list args;
    va_start(args, pstrFormat);

    str.FormatV(pstrFormat, args);
    va_end(args);

    str += L"\r\n\r\n";
    str += ErrorMessage(hr);

    uMbIconType &= MB_ICONMASK;
    HWND w = ( NULL == pWnd ) ? NULL : pWnd->m_hWnd;
    ::MessageBox( w, str, L"", MB_OK | uMbIconType );
}

inline BYTE F2B( const float f )
{
	return BYTE( std::clamp<ptrdiff_t>( ptrdiff_t( f * 256.0f ), 0, 255 ) );
}

// Source:
// http://en.wikipedia.org/wiki/HSL_and_HSV#Conversion_from_HSV_to_RGB
DWORD RgbFromHsv_f( float h, float s, float v )
{
	const int iHue = static_cast<int>( floorf( h * 360.0f ) );

	const int H = ( iHue % 360 );
	const int i = H / 60;
	const float f = ( H % 60 ) / 60.0f;

	const float p = v * ( 1.0f - s );
	const float q = v * ( 1.0f - f * s );
	const float t = v * ( 1.0f - ( 1.0f - f ) * s );

	const float arr[6][3] = 
	{
		v, t, p, 
		q, v, p,
		p, v, t,
		p, q, v,
		t, p, v, 
		v, p, q,
	};
	const float* const rgb = arr[i];
	return  D3DCOLOR_RGBA( F2B(rgb[0]), F2B(rgb[1]), F2B(rgb[2]), 255 );
}

float HueFromRgb( DWORD dwRGB )
{
	const BYTE r = GetRValue( dwRGB ), g = GetGValue( dwRGB ), b = GetBValue( dwRGB );
	const BYTE rgb[3] = { r, g, b };

	const BYTE valMin = *std::min_element( rgb, rgb + 3 );
	const BYTE valMax = *std::max_element( rgb, rgb + 3 );
	if( valMin==valMax )
		return 0;

	const float fDiff6 = 1.0f / ( 6.0f * ( valMax - valMin ) );

	if( valMax == r )
		return fmod( ( float(g) - float(b) ) * fDiff6 + 1.0f, 1.0f );

	if( valMax == g )
		return ( float(b) - float(r) ) * fDiff6 + 1.0f / 3.0f;

	if( valMax == b )
		return ( float(r) - float(g) ) * fDiff6 + 2.0f / 3.0f;

	ASSERT( 0, "E_UNEXPECTED" );
	return 0;
}