#pragma once

typedef unsigned __int64 QWORD;

inline HRESULT GetLastHr() { return HRESULT_FROM_WIN32( GetLastError() ); }

// FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM) wrapper
__declspec(noinline) CStringW ErrorMessage(HRESULT hr);

// CString::Format, then append FormatMessage(hr), then call MessageBox.
void ReportError( CWindow* pWnd, UINT uMbIconType, HRESULT hr, LPCTSTR pstrFormat, ... );

// Wrapper around QueryPerformanceFrequency
inline QWORD GetPerfFrequency()
{
    QWORD res;
    QueryPerformanceFrequency( reinterpret_cast<LARGE_INTEGER*>( &res ) );
    ASSERT( 0 != res, "QueryPerformanceFrequency failed:\n%S", LPCTSTR(ErrorMessage(GetLastHr())));
    return res;
}

// Wrapper around QueryPerformanceCounter
inline QWORD GetPerfCounter()
{
    QWORD res;
    QueryPerformanceCounter( reinterpret_cast<LARGE_INTEGER*>( &res ) );
    return res;
}

// This hash function is more suitable for CPoint's
// then the default one (which is CDefaultElementTraits<>::Hash()) that only uses the X value.
template<>
class CElementTraits< CPoint > :
    public CPrimitiveElementTraits< CPoint >
{
public:
    static inline ULONG Hash( INARGTYPE pt )
    {
        static const ULONG iHalfBits = sizeof( ULONG ) * 8 / 2;
        // Make sure the '/Oi' (='C/C++ -> Optimization -> Enable Intrisic Functions') is ON,
        // since CElementTraits::Hash is going to be called _a lot_
        return ULONG( pt.x ) ^ ( _lrotr( pt.y, iHalfBits ) );
    }
};

// Convert HSV to RGBA. Both h, s and v must be from 0.0 to 1.0
// The alpha will be set to 255.
DWORD RgbFromHsv_f( float h, float s, float v );

float HueFromRgb( DWORD dwRGB );

// Same as above, but accepts the BYTE arguments ( ranging from 0 to 255, obviously )
inline DWORD RgbFromHsv( BYTE h, BYTE s, BYTE v )
{
	const float d = 255.0;
	return RgbFromHsv_f( h/d, s/d, v/d  );
}

// Clamp a value.
// Why there's 'std::min', 'max' but no 'clamp' ? Let's fix it.
namespace std
{
	template< class T >
	inline T clamp( T val, T valMin, T valMax  )
	{
		if( val < valMin )
			return valMin;
		if( val > valMax )
			return valMax;
		return val;
	}
}

// Release the object , ASSERTing the object is really destroyed.
// For some reason, CComPtr::Release() returns void,
// making impossible to ASSERT that the resulting value of the reference count is 0.
template<class T>
inline void ReleaseComPtr( CComPtr<T>& pComPtr )
{
    if( NULL == pComPtr )
    {
        ASSERT( 0, "ReleaseComPtr is called for an uninitialized object." );
        return;
    }
    const ULONG ulNewRefCounter = pComPtr.Detach()->Release();
    ASSERT( 0 == ulNewRefCounter, "An object is alive even after ReleaseComPtr() call." );
}