#pragma once

#ifndef UNIT_TESTS
	#ifdef _DEBUG
		#define UNIT_TESTS 1
	#else
		#define UNIT_TESTS 0
	#endif
#endif

#if UNIT_TESTS
	typedef HRESULT (*UnitTestProc)();

	extern void RegisterUnitTest( const char* strFile, const char* strName, UnitTestProc testProc );

	extern int RunUnitTests( bool bSilent );

	class CUnitTestRegistrator
	{
	public:
		inline CUnitTestRegistrator( const char* strFile, const char* strName, UnitTestProc testProc )
		{
			RegisterUnitTest( strFile, strName, testProc );
		}
		inline ~CUnitTestRegistrator(){}
	};


	// I hated the C preprocessor after I had to write the code below:
	#define CONCAT_( a, b ) a ## b
	#define CONCAT( a, b ) CONCAT_( a, b )

	// Use this macro wherever you'd like to declare a global HRESULT-returning function as a unit test
	#define UNIT_TEST( testProc )  static CUnitTestRegistrator CONCAT(s_UnitTest_, __LINE__) ( __FILE__, #testProc, &testProc )

	// Call this macro in your WinMain()
	// The silent mode just silently logs what happened,
	// while the non-silent mode traps to a debugger when it encounters the 1-st failed test.
	#define RUN_UNIT_TESTS(lpCmdLine)                                                        \
		if( 0 == _wcsicmp(lpCmdLine, L"UnitTests") ) return RunUnitTests( false );           \
		if( 0 == _wcsicmp(lpCmdLine, L"UnitTestsSilent") ) return RunUnitTests( true )

#else

	#define UNIT_TEST( testProc )
	#define RUN_UNIT_TESTS( lpCmdLine )

#endif