#include "stdafx.h"
#include "Direct3DView.h"
#include "../Math/Matrix4.hpp"

// TODO: Invert Z order, so that world's Y points down not up, to make X and Y math identical.
// TODO: use mouse cursor position as an anchor point while zooming in & out.
// TODO: bubbles paint tool.
// TODO: simulation parameters UI to change'em in the runtime (including "what to hightlight" setting)
// TODO: persistent workspace
// TODO: smooth zoom
// TODO: Ctrl+Shift+F all 'TODO:'s in this entire solution and copy-paste'em into an Excel document.

CBubblesWnd::CBubblesWnd( NVsTabs::CTabsOwner* pOwner ):
    NVsTabs::CTabViewImpl< CDxWnd >( pOwner )
{
    m_bMouseTracking = false;
    m_PrevMButton = CPoint( LPARAM(-1) );
    m_TargetZoomLevel = s_DefaultZoomLevel;
    m_fCurrZoomLevel = float( m_TargetZoomLevel );
    m_clientZeroPos = CPoint(0);
}

CBubblesWnd::~CBubblesWnd()
{
}

HRESULT CBubblesWnd::Create( CWindow* pParent )
{
    static int iSN = 1;
    CString strName;
    strName.Format( L"Direct3D View #%i", iSN );
    iSN++;

    // Unlike other view windows, the Direct3D window needs its correct rectangle inside the WM_CREATE message handler.
    if( NULL == __super::Create( pParent->m_hWnd, m_pOwnerTabsPanel->GetViewRect(),
                strName, WS_CHILD | WS_CLIPCHILDREN ) )
        return GetLastHr();

    return S_OK;
}

HRESULT CBubblesWnd::CreateResources()
{
    HRESULT hr = m_Texture.LoadResource( Device, MAKEINTRESOURCE(IDR_TEX_BUBBLE) );
    if(FAILED(hr)) return hr;

	static bool bFirst = true;
	if( bFirst )
	{
		srand(0);
		bFirst = false;
	}
	else
		srand(GetTickCount());

#if 0
		const int nBubbles = 500;
		const CSize sz( 150, 100 );
#else
    #ifdef _DEBUG
//    #if 0
		const int nBubbles = 500;
		const CSize sz( 150, 100 );
    #else
		const int nBubbles = 5000;
		const CSize sz( 640, 480 );
    #endif
#endif
    m_Model.AddRandomBubbles( nBubbles, sz );

    hr = m_Model.BufferCreate( Device, D3DPOOL_DEFAULT);
    if(FAILED(hr)) return hr;

    // Test code
    m_vbTest.Create( Device, 3 );
    {
        CVertexBufferLock<TestVertStruct> lock(m_vbTest);

        lock[0].pos = Vector3F( 0, 0, 0 );
        lock[0].color = D3DCOLOR_ARGB( 255, 255, 0, 0 );
        lock[0].tex = Vector2F( 0, 0 );

        lock[1].pos = Vector3F( 100, 0, 0 );
        lock[1].color = D3DCOLOR_ARGB( 255, 0, 255, 0 );
        lock[1].tex = Vector2F( 1, 0 );

        lock[2].pos = Vector3F( 0, 100, 0 );
        lock[2].color = D3DCOLOR_ARGB( 255, 0, 0, 255 );
        lock[2].tex = Vector2F( 0, 1 );
    }
    return S_OK;
}

void CBubblesWnd::DestroyResources()
{
    m_Model.BufferDestroy();
    m_Texture.Release();
}

void CBubblesWnd::InitScene()
{
    SetupCamera();
}

void CBubblesWnd::OnPreReset()
{
    m_Model.BufferDestroy();
}

void CBubblesWnd::OnPostReset()
{
    HRESULT hr = m_Model.BufferCreate( Device, D3DPOOL_DEFAULT);
    ASSERT( SUCCEEDED(hr), "BufferCreate failed:\n%S", LPCTSTR(ErrorMessage(hr)) );
}

HRESULT CBubblesWnd::SetupCamera()
{
    HRESULT hr = S_OK;
    Matrix4F matProj;

    const float fZoom = GetZoomLevel();

    matProj.SetOrthoLH(
        float( GetCurrentSize().cx) / fZoom ,
        float( GetCurrentSize().cy ) / fZoom,
        -100.0f, 100.0f );
    const D3DMATRIX dxmatProj = matProj;
    hr = Device->SetTransform( D3DTS_PROJECTION, &dxmatProj );
    ASSERT( SUCCEEDED(hr), "SetTransform failed.\n%S", LPCTSTR(ErrorMessage(hr)));

    const Vector3F vTrans( float( m_clientZeroPos.x), float( -m_clientZeroPos.y), 0.0f );
    D3DMATRIX dxmat = Matrix4F::MakeTransition( vTrans );
    hr = Device->SetTransform( D3DTS_WORLD, &dxmat );
    ASSERT( SUCCEEDED(hr), "SetTransform failed.\n%S", LPCTSTR(ErrorMessage(hr)));

    dxmat = Matrix4F::MakeIdentity();
    hr = Device->SetTransform( D3DTS_VIEW, &dxmat );
    ASSERT( SUCCEEDED(hr), "SetTransform failed.\n%S", LPCTSTR(ErrorMessage(hr)));

    return S_OK;
}


inline DWORD dwFromF( float f )
{
    return *( reinterpret_cast< const DWORD* >( &f ) );
}

HRESULT CBubblesWnd::SetupRenderStates()
{
    HRESULT hr = S_OK;

    static const struct RS
    {
        D3DRENDERSTATETYPE rs;
        DWORD val;
    }
    arrStates[] = 
    {
        D3DRS_ZENABLE, FALSE,
        D3DRS_STENCILENABLE, FALSE,
        D3DRS_FOGENABLE, FALSE,

        D3DRS_CULLMODE, D3DCULL_NONE,
        D3DRS_LIGHTING, FALSE,

        D3DRS_POINTSPRITEENABLE, TRUE,

        // See the http://www.two-kings.de/tutorials/dxgraphics/dxgraphics17.html
        // why the { A; B; C } should be { ( wnd.cy / fZoom ) ^ 2; 0; 0 }
        D3DRS_POINTSCALEENABLE, TRUE,
        D3DRS_POINTSCALE_A, dwFromF( 1.0 ),
        D3DRS_POINTSCALE_B, dwFromF( 0 ),
        D3DRS_POINTSCALE_C, dwFromF( 0 ),

        D3DRS_ALPHABLENDENABLE, TRUE,
        D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL,
        D3DRS_ALPHAREF, (DWORD)8,
        D3DRS_ALPHATESTENABLE, TRUE,

        D3DRS_SRCBLEND, D3DBLEND_SRCALPHA,
        D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA,
    };

    for( size_t i=0; i < sizeof(arrStates) / sizeof( arrStates[0] ); i++ )
    {
        HRESULT hr = Device->SetRenderState( arrStates[i].rs, arrStates[i].val );
        ASSERT( SUCCEEDED( hr ), "SetRenderState failed." );
    }
    
    // Set that tricky D3DRS_POINTSCALE_A render state to scale the sprites properly.
    const float fZoom = GetZoomLevel();
	const CSize sz = GetCurrentSize();
	const float fSqrtA = std::min( sz.cx, sz.cy ) / fZoom;
    hr = Device->SetRenderState( D3DRS_POINTSCALE_A, dwFromF( fSqrtA * fSqrtA ) );
    ASSERT( SUCCEEDED( hr ), "SetRenderState failed." );

    return S_OK;
}

void CBubblesWnd::DrawScene( float SecsElapsed )
{
    HRESULT hr = S_OK;

    ProcessZooming( SecsElapsed );

    m_Model.SimulationStep( SecsElapsed );

    const size_t nSprites = m_Model.GetBubblesCount();
    if( nSprites )
    {
        hr = m_Model.BufferUpdate();
        ASSERT( SUCCEEDED(hr), "BufferUpdate failed.\n%S", LPCTSTR(ErrorMessage(hr)));

        hr = Device->BeginScene();
        ASSERT( SUCCEEDED(hr), "BeginScene failed.\n%S", LPCTSTR(ErrorMessage(hr)));

        SetupRenderStates();
        SetupCamera();

        hr = m_Texture.SetTexture();
        ASSERT( SUCCEEDED(hr), "SetTexture failed.\n%S", LPCTSTR(ErrorMessage(hr)));

//        m_vbTest.SetStreamSource( Device, 0 );
//        hr = Device->DrawPrimitive( D3DPT_POINTLIST, 0, 1 );
//        ASSERT( SUCCEEDED(hr), "DrawPrimitive failed.\n%S", LPCTSTR(ErrorMessage(hr)));

        hr = m_Model.BufferSetStreamSource();
        ASSERT( SUCCEEDED(hr), "BufferSetStreamSource failed.\n%S", LPCTSTR(ErrorMessage(hr)));

        hr = Device->DrawPrimitive( D3DPT_POINTLIST, 0, UINT( nSprites ) );
        ASSERT( SUCCEEDED(hr), "DrawPrimitive failed.\n%S", LPCTSTR(ErrorMessage(hr)));

        hr = Device->EndScene();
        ASSERT( SUCCEEDED(hr), "EndScene failed.\n%S", LPCTSTR(ErrorMessage(hr)));
    }
}

LRESULT CBubblesWnd::OnIdle( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
	Render();
	return LRESULT( m_Model.GetBubblesCount() + 1 );
}

Vector2D CBubblesWnd::worldFromClient( CPoint pt )
{
    Vector2D res( pt );
    res -= ( Vector2D(m_clientZeroPos) * GetZoomLevel() );
	res.x -= ( GetCurrentSize().cx / 2 );
	res.y = -res.y + ( GetCurrentSize().cy / 2 );
	return res / GetZoomLevel();
}

LRESULT CBubblesWnd::OnMouseLeave( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
    m_bMouseTracking = false;
    m_PrevMButton = CPoint( LPARAM(-1) );
    return 0;
}

LRESULT CBubblesWnd::OnMouseMsg( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
    if( !m_bMouseTracking )
    {
        TRACKMOUSEEVENT tme;
        tme.cbSize = sizeof( TRACKMOUSEEVENT );
        tme.dwFlags = TME_LEAVE;
        tme.hwndTrack = m_hWnd;
        TrackMouseEvent( &tme );
        m_bMouseTracking = true;

    }

    const CPoint ptClient = CPoint( lParam );
    const Vector2D worldPos = worldFromClient( ptClient );

    if( WM_LBUTTONDOWN == nMsg || ( WM_MOUSEMOVE == nMsg && ( wParam & MK_LBUTTON ) ) )
    {
        m_Model.MouseOperation( VK_LBUTTON, worldPos );
        return 0;
    }

    if( WM_RBUTTONDOWN == nMsg || ( WM_MOUSEMOVE == nMsg && ( wParam & MK_RBUTTON ) ) )
    {
        m_Model.MouseOperation( VK_RBUTTON, worldPos );
        return 0;
    }

    if( WM_MBUTTONDOWN == nMsg )
        m_PrevMButton = ptClient;
    else if( WM_MBUTTONUP == nMsg )
        m_PrevMButton = CPoint( LPARAM( -1 ) );
    else if( WM_MOUSEMOVE == nMsg && ( wParam & MK_MBUTTON ) )
    {
        if( m_PrevMButton.x >= 0 && m_PrevMButton.y >= 0 )
        {
            CPoint diff = ( ptClient - m_PrevMButton );
            const float fZoomFactor = 1.0f / GetZoomLevel();
            m_clientZeroPos += CPoint( int( diff.x * fZoomFactor ), int( diff.y * fZoomFactor ) );
            AdjustZeroPoint();
        }
        m_PrevMButton = ptClient;
    }

    if( WM_MOUSEWHEEL == nMsg )
    {
        const int wheelAmount = (signed __int16)(HIWORD(wParam));
        if( wheelAmount > 0 )
            SetZoomLevel( m_TargetZoomLevel + 1, ptClient );
        if( wheelAmount < 0 )
            SetZoomLevel( m_TargetZoomLevel - 1, ptClient );
        return 0;
    }

    return 0;
}

const CBubblesWnd::ZoomLevel CBubblesWnd::s_ZoomLevels[] = 
{//  MUL | DIV
    1,    3,
    1,    2,
    1,    1,   // 100%, this one should be the default
    2,    1,
    3,    1,
    4,    1,
    8,    1,
    16,   1    // 1600%
};

const int CBubblesWnd::s_DefaultZoomLevel = 2;

const int CBubblesWnd::s_ZoomLevelsCount = sizeof(CBubblesWnd::s_ZoomLevels) / sizeof(CBubblesWnd::s_ZoomLevels[0]);

void CBubblesWnd::SetZoomLevel( int iNewZoomLevel, CPoint clientAhchor )
{
    if( iNewZoomLevel < 0 )
        iNewZoomLevel = 0;
    if( iNewZoomLevel >= s_ZoomLevelsCount ) 
        iNewZoomLevel = s_ZoomLevelsCount-1;
    if( iNewZoomLevel == m_TargetZoomLevel)
        return;

    m_TargetZoomLevel = iNewZoomLevel;

//    m_ZoomLevel = iNewZoomLevel;
    //TODO: recalculate m_clientZeroPos based on the clientAhchor argument
//    AdjustZeroPoint();
}

void CBubblesWnd::AdjustZeroPoint()
{
    // Get the bounding box
    const CRect rectBubbles = m_Model.GetBoundingBox();
    if( rectBubbles.IsRectEmpty() )
        return;

    m_clientZeroPos.x = std::min( m_clientZeroPos.x, -rectBubbles.left );
    m_clientZeroPos.y = std::min( m_clientZeroPos.y, rectBubbles.bottom );
    m_clientZeroPos.x = std::max( m_clientZeroPos.x, -rectBubbles.right );
    m_clientZeroPos.y = std::max( m_clientZeroPos.y, rectBubbles.top );
}

const float CBubblesWnd::m_sZoomLevelsPerSecond = 2.0f;

float CBubblesWnd::GetIntZoomLevel( ptrdiff_t i ) const
{
    i = std::clamp<ptrdiff_t> ( i, 0, s_ZoomLevelsCount - 1 );
    return float( s_ZoomLevels[i].iMul ) / float( s_ZoomLevels[i].iDiv );
}

float CBubblesWnd::GetZoomLevel() const
{
//    return float( s_ZoomLevels[m_ZoomLevel].iMul ) / float( s_ZoomLevels[m_ZoomLevel].iDiv );

//    const size_t nZoomLevels = sizeof( s_ZoomLevels ) / sizeof( s_ZoomLevels[0] );

    const ptrdiff_t level1 = ptrdiff_t( floor(m_fCurrZoomLevel) );
    const float val1 = GetIntZoomLevel( level1 );
    const float val2 = GetIntZoomLevel( level1 + 1 );

    const float f = m_fCurrZoomLevel - float( level1 );

    return val1 * ( 1.0f - f ) + val2 * f;
}

void CBubblesWnd::ProcessZooming( float fSecsElapsed )
{
    if( m_fCurrZoomLevel == float( m_TargetZoomLevel ) )
        return;

    const float fDiff = float( m_TargetZoomLevel ) - m_fCurrZoomLevel;
    const float fDiffAbs = fabs( fDiff );
    const float fMax = fSecsElapsed * m_sZoomLevelsPerSecond;

    if( fDiffAbs <= fMax )
    {
        m_fCurrZoomLevel = float( m_TargetZoomLevel );
        return;
    }
    if( fDiff > 0 )
        m_fCurrZoomLevel += fMax;
    else
        m_fCurrZoomLevel -= fMax;
}

void CBubblesWnd::OnSize( UINT nType, CSize size )
{
    
}

void CBubblesWnd::OnChar( UINT nChar, UINT nRepCnt, UINT nFlags )
{
    if( nChar == VK_TAB )
    {
        if( !m_SettingsWnd.m_hWnd )
        {
            HRESULT hr = m_SettingsWnd.Create( this, &m_Model );
            ASSERT( SUCCEEDED(hr), "CBubbleSettingsWnd::Create failed:\n%S", LPCTSTR(ErrorMessage(hr)));
        }

        m_SettingsWnd.ShowHideWholePanel( !m_SettingsWnd.IsWindowVisible() );
    }
}
