#pragma once
#include "../VsTabs/TabViewImpl.hpp"
#include "../direct3d9/DxWnd.h"
#include "../direct3d9/Texture9.hpp"
#include "../SoapBubbleModel/SoapBubbles.h"
#include "../SoapBubbleModel/SettingsWindow.h"

// TODO: rename this 2 files to "BubblesWnd.h" and "BubblesWnd.cpp"
class CBubblesWnd:
    public NVsTabs::CTabViewImpl< CDxWnd >
{
public:
    CBubblesWnd( NVsTabs::CTabsOwner* pOwner );
    virtual ~CBubblesWnd();

    HRESULT Create( CWindow* pParent );

	static const UINT WM_IDLE = WM_USER + 11;
    
    BEGIN_MSG_MAP(CBubblesWnd)
        CHAIN_MSG_MAP( TTabViewImpl )
        CHAIN_MSG_MAP( CDxWnd )

		MESSAGE_HANDLER( WM_IDLE, OnIdle )
        MESSAGE_RANGE_HANDLER( WM_MOUSEFIRST, WM_MOUSELAST, OnMouseMsg )
        MESSAGE_HANDLER( WM_MOUSELEAVE, OnMouseLeave )
        MSG_WM_CHAR(OnChar)
        MSG_WM_SIZE(OnSize)
    END_MSG_MAP()

private:
    // CDxWnd overridables.
    // Since we only use the D3DPOOL_MANAGED resources, we don't need to handle OnPreReset and OnPostReset.

    virtual HRESULT CreateResources();

    virtual void DestroyResources();

    virtual void InitScene();

    virtual void OnPreReset();

    virtual void OnPostReset();

    virtual void DrawScene( float SecsElapsed );

	LRESULT OnIdle( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled );

	LRESULT OnMouseMsg( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled );
    LRESULT OnMouseLeave( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled );

    void OnSize( UINT nType, CSize size );
    void OnChar( UINT nChar, UINT nRepCnt, UINT nFlags );

    virtual int HitTest( CPoint pt ){ return 1; }
private:
    HRESULT SetupCamera();
    HRESULT SetupRenderStates();

    CTexture m_Texture;

    CSoapBubblesModel m_Model;

    CBubbleSettingsWnd m_SettingsWnd;

    struct TestVertStruct
    {
        Vector3F pos;
        DWORD color;
        Vector2F tex;
        static DWORD GetFVF() { return D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1; }
    };

    CVertexBuffer<TestVertStruct> m_vbTest;

	Vector2D worldFromClient( CPoint pt );

    // A coordinate of (0.0) point, relative to the center of the window, expressed in world units
    CPoint m_clientZeroPos;

    bool m_bMouseTracking;
    CPoint m_PrevMButton;

    // Available zoom levels
    struct ZoomLevel
    {
        int iMul;
        int iDiv;
    };
    static const ZoomLevel s_ZoomLevels[];
    static const int s_DefaultZoomLevel;
    static const int s_ZoomLevelsCount;

    float m_fCurrZoomLevel;
    int m_TargetZoomLevel;
    static const float m_sZoomLevelsPerSecond;

    void SetZoomLevel( int iNewZoomLevel, CPoint clientAhchor );

    // Returns some "integer" (=stable) zoom level.
    float GetIntZoomLevel( ptrdiff_t i ) const;

    // Return the currently set zoom level, as float.
    // More then 1 = magnify, less them 1 = decrease
    float GetZoomLevel() const;

    // Adjust m_clientZeroPos so that something is visible on the screen anyway.
    void AdjustZeroPoint();

    void ProcessZooming( float fSecsElapsed );

    // The functionality below is implemented in the "_Settings.cpp" source file
private:
/*
    virtual HRESULT PaintAllBubbles( PAINT_ALL_MODE mode );

    virtual HRESULT HighliteBubbles( HIGHLIGHT_MODE mode )
    {
        return m_Model.HighliteBubbles( mode );
    }
*/
};