#include "StdAfx.h"
#include "GdiViewWnd.h"

using namespace NVsTabs;

CGdiViewWnd::CGdiViewWnd( NVsTabs::CTabsOwner* pOwner ):
	CTabViewWindowImpl<CGdiViewWnd>( pOwner )
{
	static const GRADIENT_TRIANGLE ib[4] = 
	{
		{ 0, 1, 4 },
		{ 1, 2, 4 },
		{ 2, 3, 4 },
		{ 3, 0, 4 }
	};
	CopyMemory( &m_indexbuf, &ib, sizeof(ib) );

	for( size_t i=0; i<5; i++ )
	{
		m_vertexbuf[i].Alpha = 0xFFFF;
		m_vertexbuf[i].Red = WORD( rand() * 0xFFFF / RAND_MAX );
		m_vertexbuf[i].Green = WORD( rand() * 0xFFFF / RAND_MAX );
		m_vertexbuf[i].Blue = WORD( rand() * 0xFFFF / RAND_MAX );
	}
}

CGdiViewWnd::~CGdiViewWnd()
{
}

HRESULT CGdiViewWnd::Create( CWindow* pParent )
{
	static int iSN = 1;
	CString strName;
	strName.Format( L"GDI View #%i", iSN );
	iSN++;

	if( NULL == __super::Create( pParent->m_hWnd, NULL, strName, WS_CHILD ) )
		return GetLastHr();

	return S_OK;
}

LRESULT CGdiViewWnd::OnSize( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
	CRect rc;
	GetClientRect( &rc );
	static const POINT pts[ 5 ] = 
	{
		0,0, 2,0, 2,2, 0,2, 1,1
	};
	
	const CPoint ptTopLeft = rc.TopLeft();
	const CPoint sz = rc.Size();
	for( size_t i=0; i<5; i++ )
	{
		const CPoint pt( pts[i].x * sz.x / 2, pts[i].y * sz.y / 2 );
		m_vertexbuf[i].x = pt.x;
		m_vertexbuf[i].y = pt.y;
	}
	return 0;
}

LRESULT CGdiViewWnd::OnPaint( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
	PAINTSTRUCT ps;
	HDC dc = BeginPaint( &ps );
	// This implementation is not very effective.
	// The more effective way is to intersect update region with all the triangles,
	// then interpolate the colors along the edges building more triangles of less total area, only then call GradientFill.
	// This code however is IMO good enough: the flicker is acceptable,
	// and this whole .CPP is only ~60 LoC while the above-mentioned approach would require at least 2 times more.
	GradientFill( dc, m_vertexbuf, 5, m_indexbuf, 4, GRADIENT_FILL_TRIANGLE );
	EndPaint( &ps );
	return 0;
}