#include "stdafx.h"
#include "MouseProcessor.h"

CMouseProcessor::CMouseProcessor()
{
	if( !SystemParametersInfo( SPI_GETMOUSEHOVERTIME, 0, &m_MouseHoverTime, 0 ) )
		m_MouseHoverTime = 500;

	m_Timer = NULL;
	m_bMouseTracking = false;

	m_bFirst = true;
	m_bLeftBtn = m_bRightBtn = false;

    m_PrevHitTest = -1;
}

CMouseProcessor::~CMouseProcessor()
{

}

void CMouseProcessor::DoButtonUpDown( int nID, bool bDown )
{
	if( bDown )
	{
		if( m_bLeftBtn && m_idLeftBtn == nID )
			OnMouseDown( nID, VK_LBUTTON );

		if( m_bRightBtn && m_idRightBtn == nID )
			OnMouseDown( nID, VK_RBUTTON );
	}
	else
	{
		if( m_bLeftBtn && m_idLeftBtn == nID )
			OnMouseUp( nID, VK_LBUTTON );

		if( m_bRightBtn && m_idRightBtn == nID )
			OnMouseUp( nID, VK_RBUTTON );
	}
}

LRESULT CMouseProcessor::OnWmMouseLeave( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
	DoButtonUpDown( m_PrevHitTest, false );
	OnMouseInOut( m_PrevHitTest, false );
	if( m_Timer )
	{
		KillTimer( m_WindowHandle, m_Timer );
		m_Timer = NULL;
	}
	m_bMouseTracking = false;
	m_bFirst = true;
	return 0;
}

LRESULT CMouseProcessor::OnWmMouseMove( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
    const CPoint pt( lParam );
    const int iCtrl = HitTest( pt );

	if( m_bFirst )	// First mouse event here
	{
		m_bFirst = false;
		OnMouseInOut( iCtrl, true );
		DoButtonUpDown( iCtrl, true );
		m_PrevHitTest = iCtrl;

		m_MouseInTime = GetMessageTime();
		if( m_Timer )
			KillTimer( m_WindowHandle, m_Timer );
		m_Timer = SetTimer( m_WindowHandle, idTimer, 100, NULL );
	}
    else
    {
        if( m_PrevHitTest != iCtrl )
	    {
		    DoButtonUpDown( m_PrevHitTest, false );
		    OnMouseInOut( m_PrevHitTest, false );

		    if( m_Timer )
			    KillTimer( m_WindowHandle, m_Timer );
		    SetTimer( m_WindowHandle, idTimer, 100, NULL );
		    m_MouseInTime = GetMessageTime();

		    m_PrevHitTest = iCtrl;
		    OnMouseInOut( iCtrl, true );
		    DoButtonUpDown( iCtrl, true );
	    }
    }

    if( !m_bMouseTracking )
    {
        TRACKMOUSEEVENT tme;
        tme.cbSize = sizeof( TRACKMOUSEEVENT );
        tme.dwFlags = TME_LEAVE;
        tme.hwndTrack = m_WindowHandle;
        TrackMouseEvent( &tme );
        m_bMouseTracking = true;
    }

	return 0;
}

LRESULT CMouseProcessor::OnWmMouseButton( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
	const CPoint pt( lParam );
	const int iCtrl = HitTest( pt );

	if( WM_LBUTTONDOWN == nMsg )
	{
		m_bLeftBtn = true;
		m_idLeftBtn = iCtrl;
		OnMouseDown( iCtrl, VK_LBUTTON );
		return 0;
	}
		
	if( WM_RBUTTONDOWN == nMsg )
	{
		m_bRightBtn = true;
		m_idRightBtn = iCtrl;
		OnMouseDown( iCtrl, VK_RBUTTON );
		return 0;
	}

	if( WM_LBUTTONUP == nMsg )
	{
		if( !m_bLeftBtn )
			return 0;
		m_bLeftBtn = false;
		if( m_idLeftBtn != iCtrl )
			return 0;
		OnMouseUp( iCtrl, VK_LBUTTON );
		OnMouseClick( iCtrl, VK_LBUTTON );
		return 0;
	}

	if( WM_RBUTTONUP == nMsg )
	{
		if( !m_bRightBtn )
			return 0;
		m_bRightBtn = false;
		if( m_idRightBtn != iCtrl )
			return 0;
		OnMouseUp( iCtrl, VK_LBUTTON );
		OnMouseClick( iCtrl, VK_LBUTTON );
		return 0;
	}
	return 0;
}

LRESULT CMouseProcessor::OnWmTimer( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
	if( !m_MouseInTime )
	{
		KillTimer( m_WindowHandle, m_Timer );
		m_Timer = 0;
		return 0;
	}
	const DWORD dwNow = GetMessageTime();
	if( m_MouseInTime + m_MouseHoverTime < dwNow )
		return 0;

	const CPoint pt( GetMessagePos() );
	const int iCtrl = HitTest( pt );

	if( m_PrevHitTest == iCtrl )
		OnMouseHover( iCtrl, pt );
	KillTimer( m_WindowHandle, m_Timer );
	m_Timer = NULL;
	m_MouseInTime = 0;
	return 0;
}

// Below are the default event handlers, doing absolutely nothing
void CMouseProcessor::OnMouseInOut( int iCtrl, bool bIn ) { }
void CMouseProcessor::OnMouseHover( int iCtrl, CPoint pt ) { }
void CMouseProcessor::OnMouseDown( int iCtrl, DWORD vk ) { }
void CMouseProcessor::OnMouseUp( int iCtrl, DWORD vk ) { }
void CMouseProcessor::OnMouseClick( int iCtrl, DWORD vk ) { }