#pragma once
#include "TabsOwner.h"

namespace NVsTabs
{
	// Derive your view window classes from this one.
	// This class needs WM_CREATE, WM_DESTROY and WM_SETTEXT notifications, however it reverts bHandled to FALSE,
	// so please don't forget to CHAIN_MSG_MAP(TTabWindow) _at the very beginning_ of your MSG_MAP.
	template<class TBase>
	class CTabViewImpl: public TBase
	{
	public:
		typedef typename CTabViewImpl<TBase> TTabViewImpl;

		inline CTabViewImpl( CTabsOwner* pOwner ):
		m_pOwnerTabsPanel( pOwner )
		{
			m_dwTabID = 0;
		}

		virtual ~CTabViewImpl(){}

		BEGIN_MSG_MAP( CTabViewImpl )
			MESSAGE_HANDLER( WM_CREATE, OnCreate )
			MESSAGE_HANDLER( WM_DESTROY, OnDestroy )
			MESSAGE_HANDLER( WM_SETTEXT, OnSetText )
		END_MSG_MAP()
	protected:
		// Override this method if you don't want this window to have a tab.
		// To make this class re-query this value, call SetText()
		virtual bool isViewNeedsTab() { return true; }

		// Override this method to show the tooltip.
		virtual LPCTSTR GetTooltipText() { return NULL; }

        // CTabsOwner of this view window
		CTabsOwner* const m_pOwnerTabsPanel;
    private:
		// If this window has the tab, it's set to valid tab ID.
		// It's set to 0 if this view window has no tab
		DWORD m_dwTabID;

		inline bool hasTab()
		{
			return 0 != m_dwTabID;
		}

        inline bool needsTab()
        {
            return isViewNeedsTab() && TBase::GetWindowTextLength() > 0;
        }

		LRESULT OnCreate( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
		{
			bHandled = FALSE;
			if( !needsTab() )
				return 0;
			m_dwTabID = m_pOwnerTabsPanel->CreateTab( this );
			return 0;
		}

		LRESULT OnDestroy( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
		{
			bHandled = FALSE;
			if( 0 == m_dwTabID  )
				return 0;
			m_pOwnerTabsPanel->DestroyTab( m_dwTabID );
			m_dwTabID = 0;
			return 0;
		}

		LRESULT OnSetText( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
		{
			bHandled = FALSE;
			UpdateTab();
			return 0;
		}

		void UpdateTab()
		{
			const bool bHasTab   = hasTab();
			const bool bNeedsTab = needsTab();
			if( !bHasTab )
			{
				if( bNeedsTab )
					m_dwTabID = m_pOwnerTabsPanel->CreateTab( this );
				return;
			}

			if( !bNeedsTab )
			{
				m_pOwnerTabsPanel->DestroyTab( m_dwTabID );
				m_dwTabID = 0;
			}

			m_pOwnerTabsPanel->UpdateTabCaption( m_dwTabID );
		}

		virtual void OnFinalMessage( HWND hWnd )
		{
			delete this;
		}
	};

	// Just a shortcut to minimize angle braces count in the code that use that stuff.
	template<class TBase>
	class CTabViewWindowImpl:
		public CTabViewImpl< CWindowImpl<TBase, CWindow, CControlWinTraits> >
	{
	public:
		inline CTabViewWindowImpl( CTabsOwner* pOwner ):
			CTabViewImpl< CWindowImpl<TBase, CWindow, CControlWinTraits> > ( pOwner ) {}
		virtual ~CTabViewWindowImpl(){}
	};
}// namespace