#pragma once
#include "VSTabs.h"
#include "VSTabsIface.h"
#include "../Resource.h"

namespace NVsTabs
{
	// This class owns the tab panel.
	// You need to implement GetTabsRect() and GetViewRect() methods to position both tabs panel and view windows.
	// You also need to call Resize() from within the WM_SIZE message handler.

	// You might want to CHAIN_MSG_MAP from the window specified as TranslateAccelerator argument, to handle Ctrl+Tab, Ctrl+Shift+Tab and Ctrl+F4 hotkeys.
	// In this case, change s_ACCEL_TAB_NEXT, s_ACCEL_TAB_PREV and s_ACCEL_TAB_CLOSE constants below, to the corresponding 'IDM_*' constants defined in your resource files.
	class CTabsOwner:
		public IVSTabsHost
	{
	public:
		CTabsOwner();
		inline ~CTabsOwner(){}

		// Interface exposed to the CTabViewImpl:

		// Create a new tab, returns the tab ID
		DWORD CreateTab( CWindow* pView );

		// Destroy a tab
		void DestroyTab( DWORD dwTabId );

		void UpdateTabCaption( DWORD dwID );

		// Create VSTabs panel.
		HRESULT Create( CWindow* pParent, UINT uID );

		static const UINT s_ACCEL_TAB_CLOSE = ID_TAB_CLOSE;
		static const UINT s_ACCEL_TAB_NEXT = ID_TAB_NEXT;
		static const UINT s_ACCEL_TAB_PREV = ID_TAB_PREV;

		// Call from WM_SIZE message handler to property resize both tabs panel and the active view (if any).
		void Resize();

		BEGIN_MSG_MAP( CTabsOwner )
			COMMAND_ID_HANDLER( s_ACCEL_TAB_CLOSE, OnCmdTabClose )
			COMMAND_ID_HANDLER( s_ACCEL_TAB_NEXT, OnCmdTabCycleNext )
			COMMAND_ID_HANDLER( s_ACCEL_TAB_PREV, OnCmdTabCyclePrev )
		END_MSG_MAP()

        // Return the rectangle for the view windows.
        // Achtung!
        // It's your responsibility to make sure view windows don't overlap the VSTabs panel.
        virtual CRect GetViewRect() = 0;

        bool PassFocusToActiveView();
	protected:
		// Return the rectangle for the VSTabs panel.
		// The 'bottom' field is ignored since the VSTabs panel height is derived from NONCLIENTMETRICS::iMenuHeight.
		virtual CRect GetTabsRect() = 0;

		inline CWindow* GetActiveView(){ return m_pActiveView; }
	private:
		LRESULT OnCmdTabClose( WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled );
		LRESULT OnCmdTabCycleNext( WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled );
		LRESULT OnCmdTabCyclePrev( WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled );

		virtual void OnTabSwitched( DWORD dwTabID );

		virtual void OnTabContextMenu( DWORD dwTabID, CPoint pt );

		virtual void OnTabClose( DWORD dwTabID );

		inline CWindow* GetView( DWORD dwTabID )
		{
			return static_cast<CWindow*>( m_wndTabs.GetTabUserData( dwTabID ) );
		}

		CVSTabs m_wndTabs;
		CWindow* m_pActiveView;
	};
}// namespace